/*
 * fi2test: test program for "libfi2_a" library
 *
 * (c)1998 Thierry Godefroy.
 */

#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>
#include <string.h>
#include <qdos.h>
#include <fi2.h>

/* C68 global variables setting and console window definition */

char _prog_name[]   = "fi2test";
char _version[24]   = "v1.00";
char _copyright[16] = __DATE__;

char *_endmsg = "\n\nPress a key to exit.";

struct WINDOWDEF _condetails  = { 208, 1, 0, 7, 512, 256, 0, 0};
void (*_consetup)() = consetup_title;

/* Defines */

/* Normally 41 characters, but may be much more with QVFS installed */
#define FILENAME_MAX_LENGTH 255

/* Theorically up to 32767 characters but much less in practice */
#define COMMAND_STRING_MAX_LENGTH 1024

/* Useful functions */

void yes_or_no(void)
{
    char    c;

    printf("Continue (Y/N) ?\n");
    sd_cure(fgetchid(stdin), -1);

    do {
        c = getch();
        c = toupper(c);
    } while (c != 'Y' && c != 'N');

    sd_curs(fgetchid(stdin), -1);

    if (c == 'N')
        exit(0);

    printf("-----------\n");
}

void wait_for_char(void)
{
    char    c;

    sd_cure(fgetchid(stdin), -1);
    do {
        c=getch();
    } while (c == 0x00);
    sd_curs(fgetchid(stdin), -1);
}

/* Test functions */


void fi2info_test(void)
{
    int         error;
    FI2INFO_t   fi2;

    printf("fi2_info() test:\n\n");

    error = fi2_info(&fi2);

    printf("Error code    : %d\n", error);
    printf("Database Id   : %s\n", fi2.data_id);
    printf("Bitmap        : %d\n", fi2.bitmap);
    printf("Maximum key   : %d\n", fi2.max_key);
    printf("Copyright     : \n%s\n", fi2.copyright);
}


void fi2call_test(void)
{
    int     error;
    char    filename[FILENAME_MAX_LENGTH+1];
    char    parameter[256];
    char    temp[FILENAME_MAX_LENGTH+256];

    printf("fi2_call() test:\n\n");

    printf("Please give a filename: ");
    scanf("%s", filename);

    printf("Optional parameter (enter '-' for none): ");
    scanf("%s", parameter);

    strcpy(temp, filename);

    if (strcmp(parameter, "-")) {
        if (strlen(filename) & 1)
            strcat(temp, " ");
        strcat(temp, FI2_PAR_MARKER);
        strcat(temp, parameter);
    }

    error = fi2_call(-1, -1, temp);

    printf("Error code    : %d\n\n", error);
}


void fi2actn_test(void)
{
    int     error;
    char    filename[FILENAME_MAX_LENGTH+1];
    short   actions;
    void *  act_array;
    
    printf("fi2_actn() test:\n\n");

    printf("Please give a filename: ");
    scanf("%s", filename);

    error = fi2_actn(filename, &actions, &act_array);

    printf("Error code   : %d\n", error);

    if (error == 0) {

        printf("Nb of actions: %d\n",  actions);
        printf("Array address: 0x%X\n\n", (int) act_array);

        printf("You may now look at the array address with a monitor\n");
        printf("Press any key when finished, the array will then be freed");
        printf(" (fi2_free() test).\n");

        wait_for_char();

        fi2_free(act_array);
    }
}


void fi2scan_test(void)
{
    int     error;
    void *  act_array;
    short   actions;
    char    ext[5], next_ext[5];

    printf("fi2_scan() test:\n\n");

    printf("Give an extension name: ");
    scanf("%4s", ext);

    error = fi2_scan(ext, &actions, next_ext, 1, &act_array);

    printf("Error code   : %d\n", error);

    if (error == 0) {

        printf("Nb of actions: %d\n", actions);
        printf("Next ext.    : %s\n", next_ext);
        printf("Array address: 0x%X\n\n", (int) act_array);

        printf("You may now look at the array address with a monitor\n");
        printf("Press any key when finished, the array will then be freed");
        printf(" (fi2_free() test).\n");

        wait_for_char();

        fi2_free(act_array);
    }
}


void fi2parm_test(void)
{
    int                 error;
    char                ext[5], next_ext[5];
    void *              act_array;
    int                 param_code ;
    short               actions;
    int                 act_idx ;
    short               act_type;
    unsigned short      act_flags;
    char                act_param[COMMAND_STRING_MAX_LENGTH+1];

    printf("fi2_parm() test:\n\n");

    printf("Please give an extension: ");
    scanf("%4s", ext);

    error = fi2_scan(ext, &actions, next_ext, 1, &act_array);

    if (error == 0) {

        printf("What to get (0=thing, 1=exec file, 2=NULL, 3=cmd str,");
        printf(" 4=job, 5=action): ");
        scanf("%d", &param_code);

        printf("For what action (0 to %d): ", actions-1);
        scanf("%d", &act_idx);
    
        error = fi2_parm(act_array, param_code, act_idx, &act_type, \
                         &act_flags, act_param);

        fi2_free(act_array);

        printf("Error code   : %d\n", error);
        printf("Action type  : %d\n", act_type);
        printf("Action flags : 0x%X\n", act_flags);
        printf("Parameter    : %s\n\n", act_param);

    } else

        printf("Error n %d occured in fi2_scan() !\n\n", error);
}


void fi2allp_test(void)
{
    int         error;
    char        ext[5], next_ext[5];
    short       actions;
    void *      act_array;
    int         act_idx ;
    FI2ACTION_t act_bloc;
    char        act_param[COMMAND_STRING_MAX_LENGTH+1];

    printf("fi2_allp() test:\n\n");

    printf("Please give an extension: ");
    scanf("%4s", ext);

    error = fi2_scan(ext, &actions, next_ext, 1, &act_array);

    if (error == 0) {

    printf("For what action (0 to %d): ", actions-1);
    scanf("%d", &act_idx);
    
    error = fi2_allp(act_array, act_idx, &act_bloc);

    if (error == 0)
        fi2_free(act_array);

        printf("Error code   : %d\n", error);
        printf("Action type  : %d\n", act_bloc.type);
        printf("Action flags : 0x%X\n", (int) act_bloc.flags);

        qlstr_to_c(act_param, act_bloc.name);
        printf("Action name  : %s\n", act_param);

        qlstr_to_c(act_param, act_bloc.job);
        printf("Mand/Forb job: %s\n", act_param);

        qlstr_to_c(act_param, act_bloc.thing);
        printf("Thing name   : %s\n", act_param);

        qlstr_to_c(act_param, act_bloc.exec);
        printf("Execut. file : %s\n", act_param);

        qlstr_to_c(act_param, act_bloc.command);
        printf("Command str  : %s\n", act_param);

    } else

        printf("Error n %d occured in fi2_scan() !\n\n", error);
}


void fi2data_test(void)
{
    int     error;
    void *  database;

    printf("fi2_data() test:\n\n");

    error = fi2_data(0, &database);

    printf("Error code   : %d\n", error);

    if (error == 0)
        printf("Database at  : 0x%X\n\n", (int) database);
}


void fi2save_test(void)
{
    int         error;
    char        filename[FILENAME_MAX_LENGTH+1];
    chanid_t    dest_file;
    int         save_key;

    printf("fi2_save() test:\n\n");

    printf("Please give a filename: ");
    scanf("%s", filename);

    dest_file = io_open(filename, 3);

    if (dest_file > 0) {

        printf("What to save (0=all, 1=code, 2=data): ");
        scanf("%d", &save_key);

        error = fi2_save(dest_file, save_key);

        printf("Error code   : %d\n\n", error);

        io_close(dest_file);

    } else

        printf("Error %d occured while opening file !\n\n", (int) dest_file);
}


void fi2load_test(void)
{
    int         error;
    char        filename[FILENAME_MAX_LENGTH+1];
    chanid_t    data_file;
    void *      data_bloc;
    void *      old_data;

    printf("fi2_load() test:\n\n");

    printf("Please give the name of the file to load: ");
    scanf("%s", filename);

    data_file = io_open(filename, 1);

    if (data_file > 0) {

        error = fi2_load(data_file, &data_bloc, &old_data);

        printf("Error code   : %d\n", error);
        printf("New data at  : 0x%X\n", (int) data_bloc);
        printf("Old data at  : 0x%X\n\n", (int) old_data);

        io_close(data_file);

    } else

        printf("Error %d occured while opening file !\n\n", (int) data_file);
}


void fi2hctl_test(void)
{
    int     error;
    short   flags;

    printf("fi2_hctl() test:\n\n");

    printf("History flag value (-1 to read, 0=disable, 1=enable): ");
    scanf("%hd", &flags);

    error = fi2_hctl(&flags);

    printf("Error code   : %d\n", error);
    printf("Current flags: %X\n\n", flags);
}


void fi2hmax_test(void)
{
    int     error;
    short   records;

    printf("fi2_hmax() test:\n\n");

    printf("Maximum nb of records (-1 to read, 1-32767 to set): ");
    scanf("%hd", &records);

    error = fi2_hmax(&records);

    printf("Error code   : %d\n", error);
    printf("Nb of records: %d\n\n", records);
}


void fi2hget_test(void)
{
    int     error;
    short   records;
    short   record = 1;
    char    act_name[ACTION_MAX_LENGTH], filename[FILENAME_MAX_LENGTH+1];

    printf("fi2_hget() test:\n\n");

    error = fi2_hget(0, &records, NULL);

    if (error == 0) {

        if (records > 1)
            printf("There are %d records into the history\n", records);
        else
            printf("There is %d record into the history\n", records);

        if (records > 1) {

            printf("Which record are you interested in (0 to %d): ", records-1);
            scanf("%hd", &record);
        }

        if (records > 0) {

            error = fi2_hget(1, &record, act_name);

            if (error == 0)
                error = fi2_hget(2, &record, filename);

            printf("Error code   : %d\n", error);
            printf("Action name  : %s\n", act_name);
            printf("File name    : %s\n\n", filename);
        }

    } else

        printf("Error code   : %d\n\n", error);
}


void fi2htdy_test(void)
{
    int     error;

    printf("fi2_htdy() test:\n\n");

    error = fi2_htdy();

    printf("Error code   : %d\n", error);
}


void fi2hclr_test(void)
{
    int     error;

    printf("fi2_hclr() test:\n\n");

    error = fi2_hclr();

    printf("Error code   : %d\n", error);
}


void fi2hmnu_test(void)
{
    int     error;
    short   record_number;

    printf("fi2_hmnu() test:\n\n");

    error = fi2_hmnu(&record_number);

    printf("Error code   : %d\n", error);
    printf("Record number: %d\n", record_number);
}


void fi2redo_test(void)
{
    int         error;
    short       records;
    short       record  =  1;

    printf("fi2_redo() test:\n\n");

    error = fi2_hget(0, &records, NULL);

    if (error == 0) {

        if (records > 1)
            printf("There are %d records into the history\n", records);
        else
            printf("There is %d record into the history\n", records);

        if (records > 1) {

            printf("Which action do you want to redo (0 to %d): ", records-1);
            scanf("%hd", &record);
        }

        if (records > 0) {

            error = fi2_redo(-1, -1, record);

            printf("Error code   : %d\n\n", error);
        }

    } else

        printf("Error n %d occured in fi2_hget() !\n\n", error);
}


void fi2thrd_test(void)
{
    int         error;
    jobid_t     owner       = 0;
    timeout_t   delay       = 0;
    int         priority    = 8;
    char        cmd_str[COMMAND_STRING_MAX_LENGTH+1];

    printf("fi2_thrd() test:\n\n");

    printf("Please give a command string: ");
    scanf("%s", cmd_str);

    error = fi2_thrd(owner, delay, priority, cmd_str);

    printf("Error code  : %d\n", error);
}


void fi2make_fi2link_fi2unlk_test(void)
{
    int             error;
    FI2ACTION_t     act_bloc;
    char            ext[5];
    static char *   version="FIv3";

    static QLSTR_INIT(action_name,"Dummy");
    static QLSTR_INIT(job_name,"my mandatory job name");
    static QLSTR_INIT(thing_name,"my thing");
    static QLSTR_INIT(execfile_name,"my executable file");
    static QLSTR_INIT(command_str,"my command string");

    printf("fi2_make() test:\n\n");

    printf("New extension name: ");
    scanf("%4s", ext);

    act_bloc.type       =   1;
    act_bloc.flags      =   8;
    act_bloc.name       =   (QLSTR_t *) &action_name;
    act_bloc.job        =   (QLSTR_t *) &job_name;
    act_bloc.thing      =   (QLSTR_t *) &thing_name;
    act_bloc.exec       =   (QLSTR_t *) &execfile_name;
    act_bloc.command    =   (QLSTR_t *) &command_str;

    error = fi2_make(ext, version, &act_bloc);

    printf("Error code : %d\n", error);
    printf("Bloc addr  : 0x%X\n\n", (int) act_bloc.address);

    yes_or_no();

    printf("fi2_link() test:\n\n");

    error = fi2_link(NULL, act_bloc.address);

    printf("Error code : %d\n\n", error);

    yes_or_no();

    printf("fi2_unlk() test:\n\n");

    error = fi2_unlk(ext);

    printf("Error code : %d\n\n", error);
}


void main(void)
{
    fi2info_test();
    yes_or_no();

    fi2call_test();
    yes_or_no();

    fi2actn_test();
    yes_or_no();

    fi2scan_test();
    yes_or_no();

    fi2parm_test();
    yes_or_no();

    fi2allp_test();
    yes_or_no();

    fi2data_test();
    yes_or_no();

    fi2hctl_test();
    yes_or_no();

    fi2hget_test();
    yes_or_no();

    fi2htdy_test();
    yes_or_no();

    fi2hmnu_test();
    yes_or_no();

    fi2redo_test();
    yes_or_no();

    fi2thrd_test();
    yes_or_no();

    fi2hmax_test();
    yes_or_no();

    fi2hclr_test();
    yes_or_no();

    fi2save_test();
    yes_or_no();

    fi2load_test();
    yes_or_no();

    fi2make_fi2link_fi2unlk_test();
}
